import pathlib
from typing import List

import panel as pn
from panel import Template

from awesome_panel_extensions.models.resource import Resource

ROOT = pathlib.Path(__file__).parent
CSS = (ROOT / "fast_gallery_template.css").read_text()
JS = (ROOT / "fast_gallery_template.js").read_text()
TEMPLATE = (ROOT / "fast_gallery_template.html").read_text()

pn.config.raw_css.append(CSS)

class FastGalleryTemplate(Template):
    def __init__(
        self,
        items: List[Resource],
        site_name: str = "Awesome Panel",
        site_url: str = "https://awesome-panel.org",
        name: str = "Gallery",
        url: str = "https://awesome-panel.org/gallery",
        description: str = "",
        background_image_url: str = "",
        target: str = "_self",
    ):
        """[summary]

        Args:
            items (List[ApplicationMetaData]): [description]
            target (str, optional): How to open the url. One of _blank, _self, _parent or _top.
            Defaults to "_self".
        """
        JS = (ROOT / "fast_gallery_template.js").read_text()
        TEMPLATE = (ROOT / "fast_gallery_template.html").read_text()
        super().__init__(template=TEMPLATE)
        self.add_variable("title", site_name + "|" + name)
        self.add_variable("site_name", site_name)
        self.add_variable("site_url", site_url)
        self.add_variable("name", name)
        self.add_variable("url", url)
        self.add_variable("description", description)
        self.add_variable("background_image_url", background_image_url)
        self.add_variable("items", items)
        self.add_variable("gallery_js", JS)

        if not target in ["_blank", "_parent", "_top", "_self"]:
            target = "_self"
        self.add_variable("target", target)




"""The Resource contains meta data like name, description and url"""
import param

from .base_model import BaseModel
from .author import Author

class Resource(BaseModel):
    """The Resource contains meta data like name, description and url"""
    name=param.String(doc="The name")
    description = param.String(doc="A description")

    url = param.String(doc="A link to the resource")
    thumbnail_url = param.String(doc="A link to a thumbnail.")
    documentation_url = param.String(doc="A link to the documentation.")
    code_url = param.String(doc="A link to the source code.")
    video_url = param.String(doc="A link to a video.")
    gif_url = param.String(doc="A link to a .gif video")

    author = param.ClassSelector(class_=Author)

    tags = param.List(
        class_=str,
        doc="A list of tags like 'machine-learning', 'panel', 'holoviews'. Don't use spaces."
        )



import param
from .base_model import BaseModel

class Author(BaseModel):
    name = param.String(doc="The name of the author or owner of the application.")
    url = param.String(doc="A link to a page about the author.")
    avatar_url = param.String(doc="A link to a thumbnail image of the author.")

    twitter_url = param.String(doc="A link to the Twitter page of the Author")
    linkedin_url = param.String(doc="A link to the Linked In page of the Author")
    github_url = param.String(doc="A link to the Github page of the Author")


from bokeh.models.widgets import CheckboxGroup as _BkCheckboxGroup
from bokeh.core.properties import Bool, String

class FastSwitchGroup(_BkCheckboxGroup):
    readonly = Bool()
    checked_message = String()
    unchecked_message = String()